/***********************************************************************
 HAF.H
 bmajoros@tigr.org

 BOOM : Bioinformatics Object Oriented Modules

 This software is open source, and is distributed under the terms of the
 ARTISTIC LICENSE, as described at www.opensource.org.
 ***********************************************************************/
#ifndef INCL_HAF_H
#define INCL_HAF_H
#include "SubstitutionMatrix.H"
#include "BandedAlignmentMatrix.H"
#include "Alignment.H"

namespace BOOM {
class HAF
{
  float gapPenalty, frameshiftPenalty;
  int bandWidth;
  const SubstitutionMatrix<float> &M;
  const Sequence *S1;      // the reference sequence
  const Sequence *S2[3];   // the query sequence, translated in 3 frames
  Alphabet &alphabet;
  enum Direction
    {
      LEFT,
      DOWN,
      DIAGONAL
    };
  struct Cell
  {
    struct Frame
    {
      struct Link
      {
	Direction dir;
	int frame;
	void set(Direction d,int f) {dir=d; frame=f;}
      } link;
      float score;
      void set(float s,Direction d,int f) {score=s;link.set(d,f);}
    } frame[3];
  };
  BandedAlignmentMatrix<Cell> *alignmentMatrix;
  int xDim, yDim; // dimensions of alignment matrix

  void evaluateMatrix();
  void evaluateCell(HAF::Cell *thisCell,HAF::Cell *leftCell,
		    HAF::Cell *downCell,HAF::Cell *diagonalCell,
		    int x,int y);
  Alignment *traceBack(Sequence *&S2_composite);
  void followLink(const HAF::Cell::Frame::Link &link,
		  int &x,int &y,int &frame);
public:
  HAF(const Sequence &S1,const SubstitutionMatrix<float> &,
      const Sequence &S2_frame0,const Sequence &S2_frame1,
      const Sequence &S2_frame2,
      float gapPenalty,int bandWidth,float frameshiftPenalty);
  virtual ~HAF();
  Alignment *go(Sequence *&S2_composite);
};
}


#endif
