/***********************************************************************
 MultiHAF.C
 bmajoros@tigr.org

 BOOM : Bioinformatics Object Oriented Modules

 This software is open source, and is distributed under the terms of the
 ARTISTIC LICENSE, as described at www.opensource.org.
 ***********************************************************************/
#include "MultiHAF.H"
#include "AminoAlphabet.H"
using namespace BOOM;


/****************************************************************
                     MultiHAFelem methods
 ****************************************************************/
MultiHAFelem::MultiHAFelem(const Sequence *s) 
  : sequence(s),
    alignment(NULL),
    seqBookmark(0),
    alignmentBookmark(0)
{
  // ctor
}



MultiHAFelem::~MultiHAFelem() 
{
  delete alignment;
}



const Sequence *MultiHAFelem::getSequence() const
{
  return sequence;
}



const String &MultiHAFelem::getTrack() const
{
  return track;
}



void MultiHAFelem::emitGap()
{
  track+='-';
}



void MultiHAFelem::emitBase()
{
  track+=AminoAlphabet::global.lookup((*sequence)[seqBookmark]);
}



MatchType MultiHAFelem::getMatchType() const
{
  if(alignmentBookmark<0 ||
     alignmentBookmark>=alignment->getAlignmentLength()) 
    throw String("alignmentBookmark=")+alignmentBookmark+
      ", but alignmentLength="+alignment->getAlignmentLength();
  return (*alignment)[alignmentBookmark];
}



void MultiHAFelem::advanceAlignmentBookmark()
{
  ++alignmentBookmark;
}



void MultiHAFelem::advanceSeqBookmark()
{
  ++seqBookmark;
}



bool MultiHAFelem::isFullyEmitted() const
{
  return seqBookmark==sequence->getLength();
}



void MultiHAFelem::setAlignment(Alignment *alignment)
{
  this->alignment=alignment;
}



void MultiHAFelem::replaceSequence(const Sequence *newSeq)
{
  sequence=newSeq;
}

